package com.ruoyi.bst.order.service.impl;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.github.pagehelper.PageHelper;
import com.ruoyi.bst.area.domain.AreaVO;
import com.ruoyi.bst.area.service.AreaService;
import com.ruoyi.bst.bonus.domain.Bonus;
import com.ruoyi.bst.bonus.service.BonusConverter;
import com.ruoyi.bst.bonus.service.BonusService;
import com.ruoyi.bst.device.domain.DeviceVO;
import com.ruoyi.bst.device.service.DeviceIotService;
import com.ruoyi.bst.device.service.DeviceService;
import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.bo.OrderCreateBO;
import com.ruoyi.bst.order.domain.bo.OrderPayBO;
import com.ruoyi.bst.order.domain.dto.OrderCreateDTO;
import com.ruoyi.bst.order.domain.dto.OrderPayDTO;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.order.mapper.OrderMapper;
import com.ruoyi.bst.order.service.OrderConverter;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.pay.domain.Pay;
import com.ruoyi.bst.pay.domain.vo.DoPayVO;
import com.ruoyi.bst.pay.service.PayConverter;
import com.ruoyi.bst.pay.service.PayService;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.SnowFlakeUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;

/**
 * 订单Service业务层处理
 *
 * @author ruoyi
 * @date 2025-03-24
 */
@Service
public class OrderServiceImpl implements OrderService
{
    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderConverter orderConverter;

    @Autowired
    private OrderValidator orderValidator;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private BonusConverter bonusConverter;

    @Autowired
    private BonusService bonusService;

    @Autowired
    private PayService payService;

    @Autowired
    private PayConverter payConverter;

    @Autowired
    private AreaService areaService;

    @Autowired
    private DeviceIotService deviceIotService;

    /**
     * 查询订单
     *
     * @param id 订单主键
     * @return 订单
     */
    @Override
    public OrderVO selectOrderById(Long id, boolean scope)
    {
        if (id == null) {
            return null;
        }
        OrderQuery query = new OrderQuery();
        query.setId(id);
        query.setScope(scope);
        return this.selectOne(query);
    }

    @Override
    public OrderVO selectOne(OrderQuery query) {
        PageHelper.startPage(1, 1);
        List<OrderVO> list = this.selectOrderList(query);
        return CollectionUtils.firstElement(list);
    }

    /**
     * 查询订单列表
     *
     * @param order 订单
     * @return 订单
     */
    @Override
    public List<OrderVO> selectOrderList(OrderQuery order)
    {
        return orderMapper.selectOrderList(order);
    }

    /**
     * 新增订单
     *
     * @param order 订单
     * @return 结果
     */
    @Override
    public int insertOrder(Order order)
    {
        order.setCreateTime(DateUtils.getNowDate());
        order.setNo(String.valueOf(SnowFlakeUtil.newId()));
        return orderMapper.insertOrder(order);
    }

    /**
     * 修改订单
     *
     * @param order 订单
     * @return 结果
     */
    @Override
    public int updateOrder(Order order)
    {
        return orderMapper.updateOrder(order);
    }

    /**
     * 批量删除订单
     *
     * @param ids 需要删除的订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderByIds(Long[] ids)
    {
        return orderMapper.deleteOrderByIds(ids);
    }

    /**
     * 删除订单信息
     *
     * @param id 订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderById(Long id)
    {
        return orderMapper.deleteOrderById(id);
    }

    @Override
    public OrderVO createOrder(OrderCreateDTO dto) {
        ServiceUtil.assertion(dto == null || dto.getDeviceId() == null, "参数错误");

        // 加锁，防止设备重复下单
        Long lockKey = dto.getDeviceId();
        boolean lock = redisLock.lock(RedisLockKey.ORDER_CREATE, lockKey);
        ServiceUtil.assertion(!lock, "当前设备使用的人太多了，请稍后再试");
        try {
            // 转为BO对象
            OrderCreateBO bo = orderConverter.toCreateBO(dto);
            // 校验参数
            orderValidator.validate(bo);

            return transactionTemplate.execute(status -> {
                // 创建订单
                Order order = bo.getOrder();
                int rows = this.insertOrder(order);
                if (rows != 1) {
                    return null;
                }

                // 查询订单
                OrderVO vo = this.selectOrderByNo(order.getNo());

                // 更新设备当前订单ID
                int updateDevice = deviceService.updateCurrentOrderId(bo.getDevice().getId(), vo.getId());
                ServiceUtil.assertion(updateDevice != 1, "更新设备当前订单ID失败");

                // 加入延时队列，超时取消
                this.cancelWhenExpired(order);

                return vo;
            });
        } finally {
            redisLock.unlock(RedisLockKey.ORDER_CREATE, lockKey);
        }
    }

    @Override
    public OrderVO selectOrderByNo(String no, boolean scope) {
        if (StringUtils.isBlank(no)) {
            return null;
        }
        OrderQuery query = new OrderQuery();
        query.setEqNo(no);
        query.setScope(scope);
        return this.selectOne(query);
    }

    // 加入延时队列，超时取消
    private void cancelWhenExpired(Order order) {
        if (order == null || order.getPayExpireTime() == null) {
            return;
        }

        Duration duration = Duration.between(LocalDateTime.now(), order.getPayExpireTime());
        long delay = duration.getSeconds();
        if (delay <= 0) {
            delay = 1;
        }

        scheduledExecutorService.schedule(() -> {
            // 尝试取消订单
            this.cancelOrder(order.getId());
        }, delay, TimeUnit.SECONDS);
    }


    // TODO 取消订单
    @Override
    public int cancelOrder(Long id) {
        if (id == null) {
            return 0;
        }
        // TODO
        return 0;
    }


    @Override
    public DoPayVO pay(OrderVO order, OrderPayDTO dto) {
        ServiceUtil.assertion(order == null || dto == null, "参数错误");

        // 转为BO对象
        OrderPayBO bo = orderConverter.toPayBO(order, dto);
        orderValidator.validate(bo);

        // 转为分成列表
        List<Bonus> bonusList = bonusConverter.toBonusList(bo);
        ServiceUtil.assertion(CollectionUtils.isEmpty(bonusList), "转为分成列表失败");

        // 转为支付PO
        Pay pay = payConverter.toPayPO(bo);
        ServiceUtil.assertion(pay == null, "转为支付单失败");

        return transactionTemplate.execute(status -> {
            // 保存分成列表
            int bonusRows = bonusService.batchInsert(bonusList);
            ServiceUtil.assertion(bonusRows != bonusList.size(), "保存分成列表失败");
    
            // 创建并调起支付
            return payService.createPayBill(pay);
        });
    }

    @Override
    public boolean endOrder(OrderVO order) {
        ServiceUtil.assertion(order == null, "参数错误，订单不存在");
        return endOrder(order.getDeviceId(), false);
    }

    private boolean endOrder(Long deviceId, boolean ignoreInParking) {
        DeviceVO device = deviceService.selectDeviceById(deviceId);
        ServiceUtil.assertion(device == null, "参数错误，设备不存在");
        // 刷新设备信息
        deviceIotService.refresh(device, null);

        // 查询该设备全部正在使用的订单
        OrderQuery query = new OrderQuery();
        query.setStatusList(OrderStatus.inUse());
        query.setDeviceId(device.getId());
        List<OrderVO> orderList = this.selectOrderList(query);
        if (CollectionUtils.isEmptyElement(orderList)) {
            return true;
        }

        // 查询运营区
        AreaVO area = areaService.selectAreaById(device.getAreaId());
        ServiceUtil.assertion(area == null, "参数错误，运营区不存在");

        // 还车校验
        if (!ignoreInParking) {
            boolean inParking = false;  // TODO 是否在停车点
            boolean inArea = false;  // TODO 是否在运营区
            // 强制在停车点还车
            ServiceUtil.assertion(!inParking && area.getParkingReturn() != null && area.getParkingReturn(), "请将车辆停放在停车点内");
            // 强制在运营区内还车
            ServiceUtil.assertion(!inArea && area.getAreaOutReturn() != null && !area.getAreaOutReturn(), "请将车辆停放在运营区内");

            // 停车点外调度费
            if (!inParking) {
            }

            // 运营区外调度费
            if (!inArea) {
            }
        }

        // 判断是否需要审核，若需要审核，则状态为待审核

        // 若不需要审核，则为已完成

        // 若已完成，则处理订单完成

        // 订单完成需要处理：
        // 1. 更新订单状态
        // 2. 更新设备状态
        // 3. 

    }
    

}
